/*
    Mines

    Copyright (c) 1979, 2005-2011 James L. Dean

    Version 2011.02.26.01 released 2/26/2011

    This C program and associated executable may be distributed or used without
payment to its author.  Derivative works must credit its author.

    "Mines" lets you explore mines. The mine you explore is determined by a
mine number specified at the beginning of a game.

    The object of a game is to visit all of the rooms and return all of the
treasures to the entrance without making too many moves.

    In a mine, the passages are straight.  So, for example, if you go North to
leave a room, you can go South to reenter it.  The rooms are not evenly spaced.
However, the distance between adjacent rooms is always a multiple of the minimum
distance between adjacent rooms.
*/

#include <hpgcc49.h>

#define NUM_ROOMS        99
#define NUM_TREASURES    15

#define SENTENCE_LEN  1024 /* must be larger than the length of the longest sentence */
#define PARAGRAPH_LEN 8192 /* must be at least 5*SENTENCE_LEN */

typedef char      int8;
typedef short int int16;
typedef int       intC;
typedef int       int32;

typedef struct RoomNode
          {
            char  *szDescription;
            int16 nChokepoint;
            int8  bMined;
            int8  bVisited;
            union
              {
                struct
                  {
                    int16 pPassageNorth;
                    int16 pPassageSouth;
                    int16 pPassageEast;
                    int16 pPassageWest;
                    int16 pPassageUp;
                    int16 pPassageDown;
                  }                Passage;
                int16 pPassage [6];
              }   Adjacent;
           } RoomRec;

typedef struct TreasureNode
          {
            char   *szTreasure;
            char   *szGuardian;
            char   *szWeapon;
            int8   bSlain;
            int16  pRoomTreasure;
            int16  pRoomWeapon;
          } TreasureRec;

typedef struct PassageNode
          {
            int8  bBlocked;
            int16 pTreasureGuardian;
            int16 pRoom1;
            int16 pRoom2;
          } PassageRec;

typedef struct PathStackNode
          {
            int16 pRoom;
            int16 pPassageUsedToEnterRoom;
            int8  nDirectionUsedToEnterRoom;
            int16 pNext;
          } PathStackRec;

typedef struct FindPathLocalsStackNode
          {
            int8  bRoomAlreadyInStack;
            int8  nDirection1;
            int8  nDirection2;
            int8  nDirection3;
            int8  nDirectionRandom [6];
            int16 nPathLen;
            int16 pPathStack;
            int16 pNext;
          } FindPathLocalsStackRec;

typedef struct FindPathParametersStackNode
          {
            int16 pRoom;
            int8  nDirectionUsedToEnterRoom;
            int8  nReturnAddress;
            int16 pNext;
          } FindPathParametersStackRec;

static void  Carry(int16,int8,int8 *);
static intC  Command(void);
static void  DisplayHelp(int8 *);
static void  DisplayParagraph(char *);
static void  DisplayText(char *,int8 *);
static void  ExcavateMine(int16,int16,int8,int16 *,int16 *,int8 *);
static void  FindPathToEntrance(int16,int16,char **);
static void  HideTreasuresAndWeapons(int16,int16 *,int8);
static void  InitializeGlobals(void);
static void  Inventory(int16,int8,int8 *);
static void  IToA(int16,char *);
static void  Leave(int16,int8,int8 *);
static void  ListCommands(int8 *);
static void  ListPassages(int16,int8 *);
static void  ListTreasuresAndWeapons(int16,int8,int8 *);
       int   main(void);
static void  Move(int16 *,int8,int16 *,int8 *);
static void  PlayGame(int16,int16,int16 *,int16,int8,int8 *);
static void  Points(int16,int16,int16 *,int8,int16,int8,int8 *);
static int16 RandomNumber(void);
static intC  ReadChar(void);
static void  SeedRandomNumberGenerator(void);
static void  ShuffleRoomDescriptions(int16,char **);
static void  StrCat(char *,char *);
static void  StrCpy(char *,char *);
static int16 StrLen(char *);
static intC  ToUpper(intC);
static void  WayOut(int16,int16,int16,int16 *,int8,int8 *);
static void  WriteChar(char);

static FindPathLocalsStackRec         g_FindPathLocal [3*NUM_ROOMS-2] = {};
static FindPathParametersStackRec     g_FindPathParameter [3*NUM_ROOMS-2] = {};
static int8                           g_nAddIndex = 0;
static int8                           g_nDirectionOpposite [6] = {1,0,3,2,5,4};
static int16                          g_nFindPathLocals = 0;
static int16                          g_nFindPathParameters = 0;
static int16                          g_nLinesPerPage = 13;
static int8                           g_nLinesWritten = 0;
static int16                          g_nMaxLineLen = 32;
static int16                          g_nPassages = 0;
static int16                          g_nPaths = 0;
static int8                           g_nReplaceIndex = 0;
static int16                          g_nRooms = 0;
static int32                          g_nRN [8] = {};
static int32                          g_nRNPartialSum = 0;
static PassageRec                     g_Passage [9*NUM_ROOMS-8] = {};
static PathStackRec                   g_Path [6*NUM_ROOMS] = {};
static int16                          g_pRoomWithName [NUM_ROOMS+1] = {};
static RoomRec                        g_Room [3*NUM_ROOMS-2] = {};
static char                           *g_szDirection[6]
 = {"north","south","east","west","up","down"};
static char                           g_szMsg [SENTENCE_LEN] = {};
static char                           g_szParagraph [PARAGRAPH_LEN] = {};
static char                           *g_szRoom [NUM_ROOMS] = {};
static char                           g_szWayOut [NUM_ROOMS] = {};
static TreasureRec                    g_Treasure [NUM_TREASURES]
 = {
     {
       "bag full of gold coins",
       "gorgon",
       "mirror",
       0,
       -1,
       -1
     },
     {
       "large blue-white diamond",
       "grizzly bear",
       "pepper spray dispenser",
       0,
       -1,
       -1
     },
     {
       "sixty carat emerald",
       "vampire",
       "wooden stake",
       0,
       -1,
       -1
     },
     {
       "platinum crucifix",
       "crocodile",
       "roll of duct tape",
       0,
       -1,
       -1
     },
     {
       "string of pearls",
       "giant crayfish",
       "jar of Zatarain's Crab Boil",
       0,
       -1,
       -1
     },
     {
       "Ming vase",
       "troll",
       "elfin sword",
       0,
       -1,
       -1
     },
     {
       "list of the next ten winners of the Kentucky Derby",
       "werewolf",
       "silver bullet",
       0,
       -1,
       -1
     },
     {
       "pile of rubies",
       "harpy",
       "crossbow",
       0,
       -1,
       -1
     },
     {
       "previously undiscovered Rembrandt painting",
       "cobra",
       "flute fashioned from a dried-out pumpkin",
       0,
       -1,
       -1
     },
     {
       "ounce of antimatter in magnetic containment",
       "ferocious snail",
       "bag of salt",
       0,
       -1,
       -1
     },
     {
       "stack of silver bars",
       "vicious Doberman pinscher",
       "soporific-laced dog biscuit",
       0,
       -1,
       -1
     },
     {
       "set of ivory tusks",
       "colossal cockroach",
       "block of boric acid",
       0,
       -1,
       -1
     },
     {
       "Holland and Holland double rifle chambered for .600 Nitro Express",
       "giant",
       "slingshot",
       0,
       -1,
       -1
     },
     {
       "ancient Greek statue",
       "Trekkie",
       "recording of Leonard Nimoy singing \"Proud Mary\"",
       0,
       -1,
       -1
     },
     {
       "five kilograms of plutonium in a lead container",
       "titanic ant",
       "Fresnel lens",
       0,
       -1,
       -1
     }
   };

static void InitializeGlobals(
 void)
  {
    g_nLinesWritten=(int8) 0;
    g_nFindPathLocals=(int16) 0;
    g_nFindPathParameters=(int16) 0;
    g_nPassages=(int16) 0;
    g_nPaths=(int16) 0;
    g_nReplaceIndex=(int8) 0;
    g_nRooms=(int16) 0;
    g_nRNPartialSum=(int32) 0;
    g_szRoom[0]="You\'re in a spherical room.";
    g_szRoom[1]="You\'re in the hall of bones.  Dinosaur bones are everywhere.";
    g_szRoom[2]="You\'re in a subway tunnel.  Don't touch that third rail!";
    g_szRoom[3]="You\'re in a railroad tunnel.  A whistle wails in the distance.";
    g_szRoom[4]="You\'re in an elfin quiche factory.  The elves are out mowing spinach.";
    g_szRoom[5]="You\'re in an abandoned Plymouth plant.  Beware of Road Runners and Barracudas.";
    g_szRoom[6]="You\'re in an abandoned Dodge plant.  There is a Dart embedded in the North wall.";
    g_szRoom[7]="You\'re in a mouse's nest.  You'd best exhale; this is a small room.";
    g_szRoom[8]="You\'re in a giant crayfish hole.  An immense chicken neck is hanging from a rope.";
    g_szRoom[9]="You\'re in an abandoned coal mine.  Beware of methane.";
    g_szRoom[10]="You\'re in the hall of winds.  Presently, the wind is from the south.";
    g_szRoom[11]="You\'re in a stove pipe!";
    g_szRoom[12]="You\'re in a totally darkened room.  Although you can see nothing, the sound of dripping water echoes from the walls.";
    g_szRoom[13]="You\'re in an industrial waste site.  Hold your breath and don't touch anything.";
    g_szRoom[14]="You\'re in the warehouse for an extremely popular brand of home computer.  Tacky plastic enclosures abound.";
    g_szRoom[15]="You\'re in a hobbit's bedroom.  The hobbit does not look pleased!";
    g_szRoom[16]="You\'re in a troll sewerage processing plant.  The air quality is not good.";
    g_szRoom[17]="You\'re in a rabbit hole.  There is a jar of marmalade on a shelf in the wall.";
    g_szRoom[18]="You\'re in the giant's oven.  Fortunately, it hasn't been used for years.";
    g_szRoom[19]="You\'re in a hobbit's drying room.  Tobacco leaves abound.";
    g_szRoom[20]="You\'re in a large circular room.  It is dark in here.";
    g_szRoom[21]="You\'re in the Crystal Palace.  Quartz crystals cover the walls.";
    g_szRoom[22]="You\'re in the Emerald Room.  Green crystals cover the ceiling.";
    g_szRoom[23]="You\'re in a curtained room.";
    g_szRoom[24]="You\'re in an air conditioning duct!";
    g_szRoom[25]="You\'re in a giant kiln.  Smoke stains the walls.";
    g_szRoom[26]="You\'re in the Hall of Mists.  Wisps of white vapor rise from the floor.";
    g_szRoom[27]="You\'re in an Aztec pyramid.  A mummy lies in the northwest corner.";
    g_szRoom[28]="You\'re in the Room of the Eternal Flame.  A large natural gas flame burns in the center of the room.  Coming from the west wall you can barely hear the words, 'Fee Fye Foe Fum'.";
    g_szRoom[29]="You\'re in the Giant's store room.  You are surrounded by old rugs, books, chairs, etc.";
    g_szRoom[30]="You\'re in the Leprechaun's Treasure Room.  Unfortunately, the leprechaun's treasure was stolen years ago.";
    g_szRoom[31]="You\'re in a large tiled room.  A girl is inquiring after a rabbit.  Feel free to ignore her.";
    g_szRoom[32]="You\'re in a former nuclear test site.  There is a large pile of rubble in the center of the floor.  The walls are streaked with a multitude of colors.";
    g_szRoom[33]="You\'re in a drainage system holding tank.  Either it is no longer used, or it hasn't rained in a long time; the room is dry.";
    g_szRoom[34]="You\'re in Mayor Schiro's bomb shelter.";
    g_szRoom[35]="You\'re in a room with a waterfall.  It is not very impressive; it looks like someone left a water faucet running.";
    g_szRoom[36]="You\'re in an abandoned Neanderthal home.";
    g_szRoom[37]="You\'re in a volcanic chimney.  The air is getting warmer.";
    g_szRoom[38]="You\'re in a pit full of snakes.  Let's get out of here!!!!";
    g_szRoom[39]="You\'re in a salt dome.";
    g_szRoom[40]="You\'re in Eleanor Roosevelt's privy.  Wendell Wilkie buttons cover the wall.";
    g_szRoom[41]="You\'re in Napoleon's wine cellar.  German wines line the shelves.";
    g_szRoom[42]="You\'re in the space behind the giant's bathroom wall. Large razor blades litter the floor.  Step lightly.";
    g_szRoom[43]="You\'re in the room where all old toe nail clipping come to rest.  Used corn pads litter the floor.";
    g_szRoom[44]="You\'re in the Den of the Horta.  The walls are covered with a sticky fluid.  Don't touch it; it is extremely corrosive.";
    g_szRoom[45]="You\'re in a damp room.  A small creek runs into a crack in the West wall.";
    g_szRoom[46]="You\'re in what appears to be a NOPSI manhole.";
    g_szRoom[47]="You\'re in the cafeteria of Mammoth Cave.  The aroma of rancid corned beef assails your nostrils.";
    g_szRoom[48]="You\'re in a small room with a large table.  On the table is a bottle that says, 'Drink me.'";
    g_szRoom[49]="You\'re in a Confederate States of America bank vault.  Once worthless currency papers the walls.";
    g_szRoom[50]="You\'re in an abandoned subway station.";
    g_szRoom[51]="You\'re in a mine shaft.  In the distance you can hear seven high pitched voices singing, 'Hi Ho, Hi Ho, ...'";
    g_szRoom[52]="You\'re in a Minuteman missile silo.";
    g_szRoom[53]="You\'re in the giant's mouse trap.  Fortunately, you are small enough to escape.";
    g_szRoom[54]="You\'re in Adolph Hitler's summer bunker.";
    g_szRoom[55]="You\'re in a dwarf work site.  A sign says, \"Under construction. Enter at your own risk.\"";
    g_szRoom[56]="You\'re in the giant's refrigerator.  Dwarf bodies hang from hooks.";
    g_szRoom[57]="You\'re in the Dragon's Lair.  Slightly melted suits of armor litter the floor.";
    g_szRoom[58]="You\'re in a nuclear waste depository.  The walls glow faintly.";
    g_szRoom[59]="You\'re in Millard Fillmore's tomb.  It is dull.";
    g_szRoom[60]="You\'re in an abandoned corridor of the Strategic Air Command Headquarters.  A graffito reads, \"Beware of Bat Guano.\"";
    g_szRoom[61]="You\'re in a gnome's workshop.  Half-completed whoopee cushions line the tables.";
    g_szRoom[62]="You\'re in the Mummy's Tomb.  You've triggered some mechanism and the ceiling is slowly descending.";
    g_szRoom[63]="You\'re in the Underground Gourmet's retreat.  Twinkie and King Don wrappers are piled knee deep.";
    g_szRoom[64]="You\'re in a Hoola Hoop warehouse.  The words \"Shoop Shoop\" echo from the walls.";
    g_szRoom[65]="You\'re in the first circle of hell.  The living are not allowed here.";
    g_szRoom[66]="You\'re in the hall of the pixies.  The furniture appears to have been made from cradles.";
    g_szRoom[67]="You\'re in a sulfur mine.  Molten sulfur churns in a nearby pit. It is becoming difficult to breath.";
    g_szRoom[68]="You\'re in a fairy mushroom farm.  Brilliantly colored mushrooms cover the floor.";
    g_szRoom[69]="You\'re in an ice cave.  Along the western wall, a brontosaurus is defrosting.";
    g_szRoom[70]="You\'re in the giant's stove.  Fortunately, the giant now owns a microwave oven.";
    g_szRoom[71]="You\'re in the rib cage of a long deceased whale.";
    g_szRoom[72]="You\'re in a room with six walls.  The walls appear to be made of wax.  A loud buzzing noise can be heard.";
    g_szRoom[73]="You\'re in the tomb of a Pharaoh.  It has obviously been visited by others; the tomb is in a state of total disarray.";
    g_szRoom[74]="You\'re in a coal bin.  There is a fossilized fern stump here.";
    g_szRoom[75]="You\'re in a diamond mine.  It is uncomfortably hot here.";
    g_szRoom[76]="You\'re in the bottom of an oil well.  The walls are slick.";
    g_szRoom[77]="You\'re in the lowest level of Project Mohole.  The funding bubble burst before the earth did.";
    g_szRoom[78]="You\'re in the giant's cesspool.  Fortunately, the giant was connected to the city sewerage system years ago.";
    g_szRoom[79]="You\'re in an eighteenth century sewer.  The walls are crumbling brick.  Small alligators snap at your feet.";
    g_szRoom[80]="You\'re in the lair of a giant trapdoor spider.";
    g_szRoom[81]="You\'re in a giant gopher tunnel.";
    g_szRoom[82]="You\'re in a shell -- the sole remains of a prehistoric turtle.";
    g_szRoom[83]="You\'re in a small chamber.  The walls are made of earth.  The air smells of formic acid.  A strange squeaking noise can be heard in the distance.";
    g_szRoom[84]="You\'re in a chamber of columns.  The stalagmites and stalactites join here.";
    g_szRoom[85]="You\'re in a service tunnel.  Ducts, pipes, and cables are everywhere.";
    g_szRoom[86]="You\'re in a gas tank below an abandoned service station.  No smoking!";
    g_szRoom[87]="You\'re in a huge dark chamber.  To one side, a dark, muddy river moves sluggishly.  A sign written in ancient Greek says, \"Ferry out of order.\"";
    g_szRoom[88]="You\'re in a small chamber. It is brightly lit by a peculiar lichen growing on the walls and ceiling.  The floor is rocky and steeply sloping.  Nearby, a cold, clear creek boils into the floor and out of sight.";
    g_szRoom[89]="You\'re in the nest of a very large pack rat.  There are discarded aluminum Christmas trees, broken steel utility poles, and other shiny, worthless items here.";
    g_szRoom[90]="You\'re in a dungeon.  An iron maiden, a rack, a piano, and other machines of torture can be seen.";
    g_szRoom[91]="You\'re in the hall of bats.  Thousands of bats hang from the ceiling.  Watch your step; the droppings are quite deep in places.";
    g_szRoom[92]="You\'re in a hobgoblin's hideaway.";
    g_szRoom[93]="You\'re in an electrical substation.  A transformer is humming loudly.  Nearby, cables crackle with high voltage.";
    g_szRoom[94]="You\'re in the \"gold\" room.  The walls are covered with iron pyrite.";
    g_szRoom[95]="You\'re in a room with one of Dracula's emergency coffins.  The Count is out.";
    g_szRoom[96]="You\'re in a saltpeter mine.  To one side there is a huge wooden evaporation vat.  Small crystals of saltpeter cause the walls to sparkle.";
    g_szRoom[97]="You\'re in the basement cafeteria of a local hospital.  Some say that there has been nothing edible here for years.";
    g_szRoom[98]="You\'re in a troll arsenal.  Kegs of gun powder surround you.";
    return;
  }


static intC ReadChar(
 void)
  {
    intC nResult;
#ifdef HPAPINE
    char szLine [256];

    gets(&szLine[0]);
    nResult=(char) (szLine[0]);
#else
    switch (KEYCODE(keyb_getkeyM(1)))
      {
        case KB_N:
          nResult=(intC) 'N';
          break;
        case KB_S:
          nResult=(intC) 'S';
          break;
        case KB_E:
          nResult=(intC) 'E';
          break;
        case KB_W:
          nResult=(intC) 'W';
          break;
        case KB_U:
          nResult=(intC) 'U';
          break;
        case KB_D:
          nResult=(intC) 'D';
          break;
        case KB_C:
          nResult=(intC) 'C';
          break;
        case KB_I:
          nResult=(intC) 'I';
          break;
        case KB_L:
          nResult=(intC) 'L';
          break;
        case KB_P:
          nResult=(intC) 'P';
          break;
        case KB_O:
          nResult=(intC) 'O';
          break;
        case KB_Q:
          nResult=(intC) 'Q';
          break;
        case KB_H:
          nResult=(intC) 'H';
          break;
        default:
          nResult=(intC) 'X';
          break;
      }
#endif
    return nResult;
  }

static void WriteChar(
 char c)
  {
    putchar((int) c);
    return;
  }

static intC ToUpper(
 intC n)
  {
    intC nResult;

    if (n >= (intC) 'a')
      if (n <= (intC) 'z')
        nResult=(n-((intC) 'a'))+((intC) 'A');
      else
        nResult=n;
    else
      nResult=n;
    return nResult;
  }

static int16 StrLen(
 char *sz)
  {
    int16 nResult;
    char  *pc;

    nResult=0;
    pc=sz;
    while (*pc)
      {
        ++nResult;
        ++pc;
      }
    return nResult;
  }

static void StrCpy(
 char *szDestination,
 char *szSource)
  {
    char *pcDestination;
    char *pcSource;

    pcDestination=szDestination;
    pcSource=szSource;
    while (*pcSource)
      {
        *pcDestination=*pcSource;
        ++pcDestination;
        ++pcSource;
      }
    *pcDestination='\0';
    return;
  }

static void StrCat(
 char *szDestination,
 char *szSource)
  {
    char *pcDestination;

    pcDestination=szDestination;
    while (*pcDestination)
      ++pcDestination;
    StrCpy(pcDestination,szSource);
    return;
  }

static void IToA(
 int16 n,
 char  *sz)
  {
    char  c [10];
    char  cDigit [10] = {'0','1','2','3','4','5','6','7','8','9'};
    int8  nChars;
    int16 nDividend;
    int16 nQuotient;
    int16 nRemainder;
    char  *pc;

    nChars=0;
    nDividend=n;
    do
      {
        nQuotient=nDividend/10;
        nRemainder=nDividend-10*nQuotient;
        c[nChars]=cDigit[nRemainder];
        ++nChars;
        nDividend=nQuotient;
      }
    while (nDividend);
    pc=sz;
    while (nChars)
      {
        --nChars;
        *pc=c[nChars];
        ++pc;
      }
    *pc='\0';
    return;
  }

static void DisplayParagraph(
 char *szText)
  {
    int8  nColumn;
    int16 nPendingSpaces;
    int16 nTokenLen;
    int16 nTokenStart;
    int16 nTokenStop;

    nColumn=1;
    nPendingSpaces=0;
    nTokenStart=0;
    while (szText[nTokenStart] && (szText[nTokenStart] == ' '))
      {
        WriteChar(' ');
        if (++nColumn > g_nMaxLineLen)
          {
            WriteChar('\n');
            nColumn=1;
            if (++g_nLinesWritten >= (g_nLinesPerPage-1))
              {
                printf("(press a key to continue)");
                ReadChar();
                clear_screen();
                g_nLinesWritten=0;
              }
          }
        ++nTokenStart;
      }
    while (szText[nTokenStart])
      {
        while (szText[nTokenStart] && (szText[nTokenStart] == ' '))
          ++nTokenStart;
        if (nColumn+nPendingSpaces > g_nMaxLineLen)
          {
            WriteChar('\n');
            nPendingSpaces=0;
            nColumn=1;
            if (++g_nLinesWritten >= (g_nLinesPerPage-1))
              {
                printf("(press a key to continue)");
                ReadChar();
                clear_screen();
                g_nLinesWritten=0;
              }
          }
        else
          while (nPendingSpaces)
            {
              WriteChar(' ');
              ++nColumn;
              --nPendingSpaces;
            }
        if (szText[nTokenStart])
          {
            nTokenStop=nTokenStart;
            while (szText[nTokenStop] && (szText[nTokenStop] != ' '))
              ++nTokenStop;
            nTokenLen=nTokenStop-nTokenStart;
            if (nColumn+nTokenLen > g_nMaxLineLen)
              if (nTokenLen > g_nMaxLineLen)
                {
                  while (nColumn < g_nMaxLineLen)
                    {
                      WriteChar(szText[nTokenStart]);
                      ++nColumn;
                      ++nTokenStart;
                    }
                  WriteChar('\n');
                  if (++g_nLinesWritten >= (g_nLinesPerPage-1))
                    {
                      printf("(press a key to continue)");
                      ReadChar();
                      clear_screen();
                      g_nLinesWritten=0;
                    }
                  nColumn=1;
                  while (nTokenStart < nTokenStop)
                    {
                      WriteChar(szText[nTokenStart]);
                      if (++nColumn > g_nMaxLineLen)
                        {
                          WriteChar('\n');
                          if (++g_nLinesWritten >= (g_nLinesPerPage-1))
                            {
                              printf("(press a key to continue)");
                              ReadChar();
                              clear_screen();
                              g_nLinesWritten=0;
                            }
                          nColumn=1;
                        }
                      ++nTokenStart;
                    }
                }
              else
                {
                  WriteChar('\n');
                  if (++g_nLinesWritten >= (g_nLinesPerPage-1))
                    {
                      printf("(press a key to continue)");
                      ReadChar();
                      clear_screen();
                      g_nLinesWritten=0;
                    }
                  nColumn=1;
                  while (nTokenStart < nTokenStop)
                    {
                      WriteChar(szText[nTokenStart]);
                      ++nColumn;
                      ++nTokenStart;
                    }
                }
            else
              while (nTokenStart < nTokenStop)
                {
                  WriteChar(szText[nTokenStart]);
                  ++nColumn;
                  ++nTokenStart;
                }
            switch (szText[nTokenStop-1])
              {
                case '.':
                  nPendingSpaces=2;
                  break;
                case '?':
                  nPendingSpaces=2;
                  break;
                case '!':
                  nPendingSpaces=2;
                  break;
                default:
                  nPendingSpaces=1;
                  break;
              }
          }
      }
    if (nColumn > 1)
      WriteChar('\n');
    return;
  }

static void DisplayText(
 char *szText,
 int8 *bErr)
  {
    int16 nBytes;

    nBytes=StrLen(&g_szParagraph[0]);
    if (g_szParagraph[0])
      nBytes+=StrLen(" ");
    nBytes+=StrLen(szText);
    if (nBytes < PARAGRAPH_LEN)
      {
        if (g_szParagraph[0])
          StrCat(&g_szParagraph[0]," ");
        StrCat(&g_szParagraph[0],szText);
      }
    else
      {
        *bErr=TRUE;
        StrCpy(&g_szParagraph[0],"Fatal error:  the paragraph buffer is too small.");
      }
    return;
  }

static void ListCommands(
 int8 *bErr)
  {
    DisplayText(
     "Commands in the game are \"N\", \"E\", \"S\", \"W\", \"U\", and "
     "\"D\" to move North, East, South, West, Up, or Down, "
     "respectively.  Other commands are \"C\" to carry things, \"I\" to "
     "inventory what you are carrying, \"L\" to leave treasures, \"P\" to "
     "get the points you\'ve scored, \"O\" for help getting out of the "
     "mine, \"H\" for help, and \"Q\" to quit.",bErr);
    return;
  }

static void DisplayHelp(
 int8 *bErr)
  {
    ListCommands(bErr);
    if (! *bErr)
      DisplayText(
       "In a mine, the passages are straight.  So, for example, if "
       "you go North to leave a room, you can go South to reenter it.  "
       "The rooms are not evenly spaced.  However, the distance between "
       "adjacent rooms is always a multiple of the minimum distance "
       "between adjacent rooms.",bErr);
    return;
  }

static int16 RandomNumber(
 void)
  {
    int32 nResult;
   /*
       Each pseudo-random number is the modulo sum of the
     previous eight pseudo-random numbers.  A prime modulus
     makes it likely that the pseudo-random numbers will be
     uniformly distributed.  To speed computation, a partial
     sum of 7 of the 8 previous pseudo-random numbers is maintained.
          For a given set of initial values g_nRN[i], i=0,1,2,...,7,
     this random number generator should produce the same sequence
     of random numbers, no matter what 32-bit C Compiler it is compiled
     under.
   */
    do
      {
        nResult=g_nRNPartialSum+g_nRN[g_nAddIndex];
        if (nResult >= 32771) nResult-=32771;
        g_nRNPartialSum=nResult-g_nRN[g_nReplaceIndex];
        if (g_nRNPartialSum < 0) g_nRNPartialSum+=32771;
        g_nRN[g_nReplaceIndex]=nResult;
        g_nAddIndex=g_nReplaceIndex;
        if (++g_nReplaceIndex >= 8)
          g_nReplaceIndex=0;
      }
    while (nResult > 32767);
    return (int16) nResult;
  }

static void SeedRandomNumberGenerator(
 void)
  {
    int16 nKeyIndex;
    int16 nKeyStart;
    int16 nKeyLength;
    char  *pc;
    char  szLine [256];
    char  szMine [9];

    nKeyLength=0;
    clear_screen();
    g_nLinesWritten=0;
    g_szParagraph[0]='\0';
    printf("Mine number? ");
    gets(&szLine[0]);
    pc=&szLine[0];
    while ((*pc) && (nKeyLength < 8))
      {
        szMine[nKeyLength]=*pc;
        ++nKeyLength;
        ++pc;
      }
    WriteChar('\n');
    szMine[nKeyLength]='\0';
    nKeyIndex=0;
    if (nKeyLength < 8)
      {
        for (nKeyStart=nKeyLength; nKeyStart < 8; ++nKeyStart)
          g_nRN[nKeyIndex++]=1+(int) '0';
        pc=szMine;
      }
    else
      pc=szMine+nKeyLength-8;
    while (*pc)
      g_nRN[nKeyIndex++]=1+(int) *pc++;
    g_nRNPartialSum=0;
    for (nKeyIndex=7; nKeyIndex > 0; --nKeyIndex)
      {
        g_nRNPartialSum+=g_nRN[nKeyIndex];
        if (g_nRNPartialSum >= 32771)
          g_nRNPartialSum-=32771;
      }
    g_nReplaceIndex=1;
    g_nAddIndex=0;
    for (nKeyIndex=256; nKeyIndex--;)
      RandomNumber();
    return;
  }

static void ShuffleRoomDescriptions(
 int16 nRooms,
 char  **pszRoom)
  {
    int16 nRoom1;
    int16 nRoom2;
    char  *sz;

    for (nRoom1=nRooms-1; nRoom1 > 0; -- nRoom1)
      {
        nRoom2=RandomNumber()%nRoom1;
        sz=pszRoom[nRoom1];
        pszRoom[nRoom1]=pszRoom[nRoom2];
        pszRoom[nRoom2]=sz;
      }
    return;
  }

static void ExcavateMine(
 int16 nRooms,
 int16 pRoomEntrance,
 int8  nTreasures,
 int16 *nChokepoints,
 int16 *ppRoomWithName,
 int8  *bErr)
  {
    int16 nChokepoint;
    int8  nDirectionOpposite;
    int8  nDirection1;
    int8  nDirection2;
    int16 nDistance;
    int8  nHeight;
    int8  nLen;
    int8  nMax;
    int16 nRoom;
    int16 nStep;
    int8  nTreasure;
    int16 nUnblocked;
    int8  nWidth;
    int8  nX;
    int8  nXFirstRoom;
    int8  nY;
    int8  nYFirstRoom;
    int8  nZ;
    int8  nZFirstRoom;
    int16 pPassage;
    int16 pRoom;
    int16 pRoomDown;
    int16 pRoomNext;
    int16 pRoomStart;
    int16 pRoomSouth;
    int16 pRoomSouthBase;
    int16 pRoomWest;
    int16 pRoomWestBase;
    int16 pRoomWestBaseBase;
    int32 nRN;

    /* nMax=(int) (10.0+exp(log((double) nRooms)/3.0)); */
    nMax=0;
    while (nMax*nMax*nMax < nRooms)
      ++nMax;
    if (nMax*nMax*nMax == nRooms)
      nMax+=10;
    else
      nMax+=9;
    do
      {
        nRN=RandomNumber();
        nLen=nRN%(nMax-3)+3;
        nRN=RandomNumber();
        nWidth=nRN%(nMax-3)+3;
        nRN=RandomNumber();
        nHeight=nRN%(nMax-3)+3;
      }
    while ((nLen*nWidth*nHeight < 2*(nRooms-1))
    ||     (nLen*nWidth*nHeight > 3*(nRooms-1)));
    nXFirstRoom=0;
    nYFirstRoom=RandomNumber()%nLen;
    nZFirstRoom=RandomNumber()%nHeight;
    pRoomWestBase=-1;
    pRoomWestBaseBase=-1;
    for (nX=0; ((! *bErr) && (nX < nWidth)); ++nX)
      {
        pRoomSouthBase=-1;
        for (nY=0; ((! *bErr) && (nY < nLen)); ++nY)
          {
            pRoomWest=pRoomWestBase;
            pRoomSouth=pRoomSouthBase;
            pRoomDown=-1;
            for (nZ=0; ((! *bErr) && (nZ < nHeight)); ++nZ)
              {
                if (g_nRooms < 3*NUM_ROOMS-2)
                  {
                    pRoom=g_nRooms;
                    ++g_nRooms;
                    g_Room[pRoom].szDescription=NULL;
                    g_Room[pRoom].bMined=FALSE;
                    g_Room[pRoom].nChokepoint=-1;
                    g_Room[pRoom].bVisited=FALSE;
                    g_Room[pRoom].Adjacent.Passage.pPassageNorth=-1;
                    g_Room[pRoom].Adjacent.Passage.pPassageSouth=-1;
                    g_Room[pRoom].Adjacent.Passage.pPassageEast=-1;
                    g_Room[pRoom].Adjacent.Passage.pPassageWest=-1;
                    g_Room[pRoom].Adjacent.Passage.pPassageUp=-1;
                    g_Room[pRoom].Adjacent.Passage.pPassageDown=-1;
/*
(X,Y,Z) gets linked to (X-1,Y,Z); (X,Y-1,Z); and (X,Y,Z-1)
                       pRoomWest  pRoomSouth     pRoomDown
*/
                    if (nZ == 0)
                      {
                        pRoomSouthBase=pRoom;
                        if (nY == 0)
                          pRoomWestBaseBase=pRoom;
                      }
                    if (pRoomDown != -1)
                      {
                      /* link room to one below it */
                        if (g_nPassages < (9*NUM_ROOMS-8))
                          {
                            pPassage=g_nPassages;
                            ++g_nPassages;
                            g_Passage[pPassage].bBlocked=TRUE;
                            g_Passage[pPassage].pTreasureGuardian=-1;
                            g_Passage[pPassage].pRoom1=pRoom;
                            g_Passage[pPassage].pRoom2=pRoomDown;
                            g_Room[pRoom].Adjacent.Passage.pPassageDown=pPassage;
                            g_Room[pRoomDown].Adjacent.Passage.pPassageUp=pPassage;
                          }
                        else
                          {
                            DisplayText("Fatal error:  memory was exhausted at locaton 1.",bErr);
                            *bErr=TRUE;
                          }
                      }
                    if (pRoomSouth != -1)
                      {
                        /* link to room to the south */
                        if (g_nPassages < 9*NUM_ROOMS-8)
                          {
                            pPassage=g_nPassages;
                            ++g_nPassages;
                            g_Passage[pPassage].bBlocked=TRUE;
                            g_Passage[pPassage].pTreasureGuardian=-1;
                            g_Passage[pPassage].pRoom1=pRoom;
                            g_Passage[pPassage].pRoom2=pRoomSouth;
                            g_Room[pRoom].Adjacent.Passage.pPassageSouth=pPassage;
                            g_Room[pRoomSouth].Adjacent.Passage.pPassageNorth=pPassage;
                          }
                        else
                          {
                            DisplayText("Fatal error:  memory was exhausted at locaton 2.",bErr);
                            *bErr=TRUE;
                          }
                        if (g_Room[pRoomSouth].Adjacent.Passage.pPassageUp != -1)
                          if (g_Passage[g_Room[pRoomSouth].Adjacent.Passage.pPassageUp].pRoom1
                           == pRoomSouth)
                            pRoomSouth=g_Passage[g_Room[pRoomSouth].Adjacent.Passage.pPassageUp].pRoom2;
                          else
                            pRoomSouth=g_Passage[g_Room[pRoomSouth].Adjacent.Passage.pPassageUp].pRoom1;
                        else
                          pRoomSouth=-1;
                      }
                    if (pRoomWest != -1)
                      {
                      /* link to the room to the west */
                        if (g_nPassages < 9*NUM_ROOMS-8)
                          {
                            pPassage=g_nPassages;
                            ++g_nPassages;
                            g_Passage[pPassage].bBlocked=TRUE;
                            g_Passage[pPassage].pTreasureGuardian=-1;
                            g_Passage[pPassage].pRoom1=pRoom;
                            g_Passage[pPassage].pRoom2=pRoomWest;
                            g_Room[pRoom].Adjacent.Passage.pPassageWest=pPassage;
                            g_Room[pRoomWest].Adjacent.Passage.pPassageEast=pPassage;
                          }
                        else
                          {
                            DisplayText("Fatal error:  memory was exhausted at locaton 3.",bErr);
                            *bErr=TRUE;
                          }
                        if (g_Room[pRoomWest].Adjacent.Passage.pPassageUp != -1)
                          if (g_Passage[g_Room[pRoomWest].Adjacent.Passage.pPassageUp].pRoom1
                           == pRoomWest)
                            pRoomWest=g_Passage[g_Room[pRoomWest].Adjacent.Passage.pPassageUp].pRoom2;
                          else
                            pRoomWest=g_Passage[g_Room[pRoomWest].Adjacent.Passage.pPassageUp].pRoom1;
                        else
                          pRoomWest=-1;

                      }
                    /* If this is the first room, connect it to the entrance. */
                    if ((nX == nXFirstRoom)
                    &&  (nY == nYFirstRoom)
                    &&  (nZ == nZFirstRoom))
                      {
                        if (g_nPassages < 9*NUM_ROOMS-8)
                          {
                            pPassage=g_nPassages;
                            ++g_nPassages;
                            g_Passage[pPassage].bBlocked=FALSE;
                            g_Passage[pPassage].pTreasureGuardian=-1;
                            g_Passage[pPassage].pRoom1=pRoom;
                            g_Passage[pPassage].pRoom2=pRoomEntrance;
                            g_Room[pRoom].Adjacent.Passage.pPassageWest=pPassage;
                            g_Room[pRoomEntrance].Adjacent.Passage.pPassageEast
                             =pPassage;
                          }
                        else
                          {
                            DisplayText("Fatal error:  memory was exhausted at locaton 4.",bErr);
                            *bErr=TRUE;
                          }
                      }
                    pRoomDown=pRoom;
                  }
                else
                  {
                    DisplayText("Fatal error:  memory was exhausted at locaton 5.",bErr);
                    *bErr=TRUE;
                  }
              }
            if (pRoomWestBase != -1)
              {
                if (g_Room[pRoomWestBase].Adjacent.Passage.pPassageNorth != -1)
                  {
                    if (g_Passage[g_Room[pRoomWestBase].Adjacent.Passage.pPassageNorth].pRoom1
                     == pRoomWestBase)
                      pRoomWestBase=g_Passage[g_Room[pRoomWestBase].Adjacent.Passage.pPassageNorth].pRoom2;
                    else
                      pRoomWestBase=g_Passage[g_Room[pRoomWestBase].Adjacent.Passage.pPassageNorth].pRoom1;
                  }
              }
          }
        pRoomWestBase=pRoomWestBaseBase;
      }
    nChokepoint=-1;
    nTreasure=0;
    pRoom=pRoomEntrance;
    nRoom=0;
    while (nRoom < nRooms)
      {
        pRoomStart=pRoom;
        nDirection1=RandomNumber()%6;
        nDirectionOpposite=g_nDirectionOpposite[nDirection1];
        nDistance=RandomNumber()%3+1;
        nStep=0;
        while ((g_Room[pRoom].Adjacent.pPassage[nDirection1] != -1)
        &&     (nStep < nDistance))
          {
            pPassage=g_Room[pRoom].Adjacent.pPassage[nDirection1];
            if (g_Passage[pPassage].pRoom1 == pRoom)
              pRoomNext=g_Passage[pPassage].pRoom2;
            else
              pRoomNext=g_Passage[pPassage].pRoom1;
            if (g_Passage[pPassage].bBlocked)
              if ((g_Room[pRoomNext].nChokepoint == nChokepoint)
              ||  (! g_Room[pRoomNext].bMined))
                {
                  pRoom=pRoomNext;
                  g_Room[pRoom].bMined=TRUE;
                  g_Room[pRoom].nChokepoint=nChokepoint;
                  g_Passage[pPassage].bBlocked=FALSE;
                  if (g_Room[pRoom].szDescription)
                    ++nStep;
                  else
                    {
                      nUnblocked=0;
                      for (nDirection2=6;
                       ((nUnblocked < 2) && (nDirection2--));)
                        if (nDirection2 != nDirectionOpposite)
                          if (g_Room[pRoom].Adjacent.pPassage[nDirection2] != -1)
                            if (! g_Passage[g_Room[pRoom].Adjacent.pPassage[nDirection2]].bBlocked)
                              ++nUnblocked;
                      if (nUnblocked < 2)
                        ++nStep;
                      else
                        nStep=nDistance;
                    }
                }
              else
                nStep=nDistance;
            else
              {
                ++nStep;
                pRoom=pRoomNext;
                nChokepoint=g_Room[pRoom].nChokepoint;
              }
          }
        if (nStep)
          {
            if (g_Room[pRoom].szDescription == NULL)
              {
                g_Room[pRoom].szDescription=g_szRoom[nRoom];
                nDirection1=0;
                nUnblocked=0;
                for (nDirection2=6;
                 ((nUnblocked < 2) && (nDirection2--));)
                  if (g_Room[pRoom].Adjacent.pPassage[nDirection2] != -1)
                    if (! g_Passage[g_Room[pRoom].Adjacent.pPassage[nDirection2]].bBlocked)
                      {
                        nDirection1=nDirection2;
                        ++nUnblocked;
                      }
                if (nUnblocked == 1)
                  {
                    if (((int32) (nRooms-nRoom))*RandomNumber() < 32768*((int32) (nTreasures-nTreasure)))
                      {
                        nChokepoint=nTreasure;
                        g_Passage[g_Room[pRoom].Adjacent.pPassage[nDirection1]].
                         pTreasureGuardian=nTreasure;
                        ++nTreasure;
                      }
                  }
                g_Room[pRoom].nChokepoint=nChokepoint;
                ppRoomWithName[nRoom]=pRoom;
                ++nRoom;
              }
          }
      }
    ppRoomWithName[nRoom]=pRoomEntrance;
    *nChokepoints=nTreasure;
    return;
  }

static void FindPathToEntrance(
 int16 pRoomStart,
 int16 pRoomEntrance,
 char  **pszWayOut)
  {
    int8  bOutOfMemory;
    int8  bReenter;
    int8  bResume;
    int16 nIterations;
    int8  nReturnAddress;
    int16 pFindPathLocals;
    int16 pFindPathLocalsHead;
    int16 pFindPathParameters;
    int16 pFindPathParametersHead;
    int16 pPathStackHead;

    bOutOfMemory=FALSE;
    pFindPathLocals=-1;
    pFindPathLocalsHead=-1;
    pFindPathParameters=-1;
    pFindPathParametersHead=-1;
    if (g_nFindPathParameters < 3*NUM_ROOMS-2)
      {
        pFindPathParameters=g_nFindPathParameters;
        ++g_nFindPathParameters;
        g_FindPathParameter[pFindPathParameters].pNext=pFindPathParametersHead;
        g_FindPathParameter[pFindPathParameters].pRoom=pRoomStart;
        g_FindPathParameter[pFindPathParameters].nDirectionUsedToEnterRoom=0;
        g_FindPathParameter[pFindPathParameters].nReturnAddress=0;
        pFindPathParametersHead=pFindPathParameters;
        pPathStackHead=-1;
        bResume=FALSE;
        nIterations=0;
        do
          {
            ++nIterations;
            bReenter=FALSE;
            if (bResume)
              {
                for (;
                 ((! bOutOfMemory)
                 && (! bReenter)
                 && (! *pszWayOut)
                 && ((g_FindPathLocal[pFindPathLocalsHead].nDirection1) > 0));)
                  {
                    --(g_FindPathLocal[pFindPathLocalsHead].nDirection1);
                    g_FindPathLocal[pFindPathLocalsHead].nDirection2
                     =(g_FindPathLocal[pFindPathLocalsHead].nDirectionRandom)[
                     g_FindPathLocal[pFindPathLocalsHead].nDirection1];
                    if (g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom].Adjacent.pPassage[
                     g_FindPathLocal[pFindPathLocalsHead].nDirection2] != -1)
                      {
                        if (! g_Passage[g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom].
                         Adjacent.pPassage[
                         g_FindPathLocal[pFindPathLocalsHead].nDirection2]].bBlocked)
                          {
                            if (g_Passage[g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom].
                             Adjacent.pPassage[
                             g_FindPathLocal[pFindPathLocalsHead].nDirection2]].pRoom1
                             == g_FindPathParameter[pFindPathParametersHead].pRoom)
                              if (g_nFindPathParameters < 3*NUM_ROOMS-2)
                                {
                                  pFindPathParameters=g_nFindPathParameters;
                                  ++g_nFindPathParameters;
                                  g_FindPathParameter[pFindPathParameters].pNext=pFindPathParametersHead;
                                  g_FindPathParameter[pFindPathParameters].pRoom
                                   =g_Passage[g_Room[g_FindPathParameter[pFindPathParametersHead
                                   ].pRoom].Adjacent.pPassage[g_FindPathLocal[pFindPathLocalsHead
                                   ].nDirection2]].pRoom2;
                                  g_FindPathParameter[pFindPathParameters].nDirectionUsedToEnterRoom=g_FindPathLocal[pFindPathLocalsHead].nDirection2;
                                  g_FindPathParameter[pFindPathParameters].nReturnAddress=1;
                                  pFindPathParametersHead=pFindPathParameters;
                                  bReenter=TRUE;
                                }
                              else
                                bOutOfMemory=TRUE;
                            else
                              if (g_nFindPathParameters < 3*NUM_ROOMS-2)
                                {
                                  pFindPathParameters=g_nFindPathParameters;
                                  ++g_nFindPathParameters;
                                  g_FindPathParameter[pFindPathParameters].pNext=pFindPathParametersHead;
                                  g_FindPathParameter[pFindPathParameters].pRoom
                                   =g_Passage[g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom
                                   ].Adjacent.pPassage[g_FindPathLocal[pFindPathLocalsHead
                                   ].nDirection2]].pRoom1;
                                  g_FindPathParameter[pFindPathParameters].nDirectionUsedToEnterRoom=g_FindPathLocal[pFindPathLocalsHead].nDirection2;
                                  g_FindPathParameter[pFindPathParameters].nReturnAddress=1;
                                  pFindPathParametersHead=pFindPathParameters;
                                  bReenter=TRUE;
                                }
                              else
                                bOutOfMemory=TRUE;
                          }
                      }
                  }
                if (! bOutOfMemory)
                  {
                    if (! bReenter)
                      {
                        pPathStackHead=g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pNext;
                        g_FindPathLocal[pFindPathLocalsHead].pPathStack=-1;
                        pFindPathLocals=pFindPathLocalsHead;
                        pFindPathLocalsHead=g_FindPathLocal[pFindPathLocals].pNext;
                        --g_nFindPathLocals;
                        pFindPathLocals=-1;
                      }
                  }
              }
            else
              bReenter=TRUE;
            if (! bOutOfMemory)
              {
                if (bReenter)
                  do
                    {
                      bReenter=FALSE;
                      if (g_nFindPathLocals < 3*NUM_ROOMS-2)
                        {
                          pFindPathLocals=g_nFindPathLocals;
                          ++g_nFindPathLocals;
                          g_FindPathLocal[pFindPathLocals].pNext=pFindPathLocalsHead;
                          pFindPathLocalsHead=pFindPathLocals;
                          if (g_FindPathParameter[pFindPathParametersHead].pRoom == pRoomEntrance)
                            {
                              g_FindPathLocal[pFindPathLocalsHead].nPathLen=0;
                              g_FindPathLocal[pFindPathLocalsHead].pPathStack=pPathStackHead;
                              while (g_FindPathLocal[pFindPathLocalsHead].pPathStack != -1)
                                {
                                  if (g_Room[g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pRoom].szDescription)
                                    ++(g_FindPathLocal[pFindPathLocalsHead].nPathLen);
                                  g_FindPathLocal[pFindPathLocalsHead].pPathStack=g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pNext;
                                }
                              if (1+(g_FindPathLocal[pFindPathLocalsHead].nPathLen) < NUM_ROOMS)
                                {
                                  *pszWayOut=&g_szWayOut[0];
                                  (*pszWayOut)[g_FindPathLocal[pFindPathLocalsHead].nPathLen]='\0';
                                  (*pszWayOut)[--(g_FindPathLocal[pFindPathLocalsHead].nPathLen)]=(char) ToUpper((intC) *g_szDirection[g_FindPathParameter[pFindPathParametersHead].nDirectionUsedToEnterRoom]);
                                  g_FindPathLocal[pFindPathLocalsHead].pPathStack=pPathStackHead;
                                  while (g_FindPathLocal[pFindPathLocalsHead].pPathStack != -1)
                                    {
                                      if (g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pNext != -1)
                                        {
                                          if (g_Room[g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pRoom].szDescription)
                                            (*pszWayOut)[--(g_FindPathLocal[pFindPathLocalsHead].nPathLen)]
                                             =(char) ToUpper((intC) *g_szDirection[g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].nDirectionUsedToEnterRoom]);
                                        }
                                      g_FindPathLocal[pFindPathLocalsHead].pPathStack=g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pNext;
                                    }
                                }
                              else
                                bOutOfMemory=TRUE;
                            }
                          else
                            {
                              g_FindPathLocal[pFindPathLocalsHead].bRoomAlreadyInStack=FALSE;
                              g_FindPathLocal[pFindPathLocalsHead].pPathStack=pPathStackHead;
                              while ((g_FindPathLocal[pFindPathLocalsHead].pPathStack != -1)
                              &&     (! (g_FindPathLocal[pFindPathLocalsHead].bRoomAlreadyInStack)))
                                {
                                  g_FindPathLocal[pFindPathLocalsHead].bRoomAlreadyInStack=(g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pRoom == g_FindPathParameter[pFindPathParametersHead].pRoom);
                                  g_FindPathLocal[pFindPathLocalsHead].pPathStack=g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pNext;
                                }
                              if (! (g_FindPathLocal[pFindPathLocalsHead].bRoomAlreadyInStack))
                                {
                                  if (g_nPaths < 6*NUM_ROOMS)
                                    {
                                      g_FindPathLocal[pFindPathLocalsHead].pPathStack=g_nPaths;
                                      ++g_nPaths;
                                      g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pRoom=g_FindPathParameter[pFindPathParametersHead].pRoom;
                                      g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].nDirectionUsedToEnterRoom=g_FindPathParameter[pFindPathParametersHead].nDirectionUsedToEnterRoom;
                                      g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pNext=pPathStackHead;
                                      pPathStackHead=g_FindPathLocal[pFindPathLocalsHead].pPathStack;
                                      for (g_FindPathLocal[pFindPathLocalsHead].nDirection1=6; (g_FindPathLocal[pFindPathLocalsHead].nDirection1)--;)
                                        (g_FindPathLocal[pFindPathLocalsHead].nDirectionRandom)[g_FindPathLocal[pFindPathLocalsHead].nDirection1]=g_FindPathLocal[pFindPathLocalsHead].nDirection1;
                                      for (g_FindPathLocal[pFindPathLocalsHead].nDirection1=5; g_FindPathLocal[pFindPathLocalsHead].nDirection1 > 0; --(g_FindPathLocal[pFindPathLocalsHead].nDirection1))
                                        {
                                          g_FindPathLocal[pFindPathLocalsHead].nDirection2=RandomNumber()%(g_FindPathLocal[pFindPathLocalsHead].nDirection1);
                                          g_FindPathLocal[pFindPathLocalsHead].nDirection3=(g_FindPathLocal[pFindPathLocalsHead].nDirectionRandom)[g_FindPathLocal[pFindPathLocalsHead].nDirection1];
                                          (g_FindPathLocal[pFindPathLocalsHead].nDirectionRandom)[g_FindPathLocal[pFindPathLocalsHead].nDirection1]=(g_FindPathLocal[pFindPathLocalsHead].nDirectionRandom)[g_FindPathLocal[pFindPathLocalsHead].nDirection2];
                                          (g_FindPathLocal[pFindPathLocalsHead].nDirectionRandom)[g_FindPathLocal[pFindPathLocalsHead].nDirection2]=g_FindPathLocal[pFindPathLocalsHead].nDirection3;
                                        }
                                      for (g_FindPathLocal[pFindPathLocalsHead].nDirection1=6; ((! bOutOfMemory) && (! bReenter) && (! *pszWayOut) && ((g_FindPathLocal[pFindPathLocalsHead].nDirection1) > 0));)
                                        {
                                          --(g_FindPathLocal[pFindPathLocalsHead].nDirection1);
                                          g_FindPathLocal[pFindPathLocalsHead].nDirection2=(g_FindPathLocal[pFindPathLocalsHead].nDirectionRandom)[g_FindPathLocal[pFindPathLocalsHead].nDirection1];
                                          if (g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom].Adjacent.pPassage[g_FindPathLocal[pFindPathLocalsHead].nDirection2] != -1)
                                            {
                                              if (! g_Passage[g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom
                                               ].Adjacent.pPassage[g_FindPathLocal[pFindPathLocalsHead].nDirection2
                                               ]].bBlocked)
                                                {
                                                  if (g_Passage[g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom
                                                   ].Adjacent.pPassage[g_FindPathLocal[pFindPathLocalsHead
                                                   ].nDirection2]].pRoom1 == g_FindPathParameter[pFindPathParametersHead].pRoom)
                                                    if (g_nFindPathParameters < 3*NUM_ROOMS-2)
                                                      {
                                                        pFindPathParameters=g_nFindPathParameters;
                                                        ++g_nFindPathParameters;
                                                        g_FindPathParameter[pFindPathParameters].pNext=pFindPathParametersHead;
                                                        g_FindPathParameter[pFindPathParameters].pRoom
                                                         =g_Passage[g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom
                                                         ].Adjacent.pPassage[g_FindPathLocal[pFindPathLocalsHead
                                                         ].nDirection2]].pRoom2;
                                                        g_FindPathParameter[pFindPathParameters].nDirectionUsedToEnterRoom=g_FindPathLocal[pFindPathLocalsHead].nDirection2;
                                                        g_FindPathParameter[pFindPathParameters].nReturnAddress=1;
                                                        pFindPathParametersHead=pFindPathParameters;
                                                        bReenter=TRUE;
                                                      }
                                                    else
                                                      bOutOfMemory=TRUE;
                                                  else
                                                    if (g_nFindPathParameters < 3*NUM_ROOMS-2)
                                                      {
                                                        pFindPathParameters=g_nFindPathParameters;
                                                        ++g_nFindPathParameters;
                                                        g_FindPathParameter[pFindPathParameters].pNext=pFindPathParametersHead;
                                                        g_FindPathParameter[pFindPathParameters].pRoom
                                                         =g_Passage[g_Room[g_FindPathParameter[pFindPathParametersHead].pRoom
                                                         ].Adjacent.pPassage[g_FindPathLocal[pFindPathLocalsHead
                                                         ].nDirection2]].pRoom1;
                                                        g_FindPathParameter[pFindPathParameters].nDirectionUsedToEnterRoom=g_FindPathLocal[pFindPathLocalsHead].nDirection2;
                                                        g_FindPathParameter[pFindPathParameters].nReturnAddress=1;
                                                        pFindPathParametersHead=pFindPathParameters;
                                                        bReenter=TRUE;
                                                      }
                                                    else
                                                      bOutOfMemory=TRUE;
                                                }
                                            }
                                        }
                                      if (! bReenter)
                                        {
                                          pPathStackHead=g_Path[g_FindPathLocal[pFindPathLocalsHead].pPathStack].pNext;
                                          g_FindPathLocal[pFindPathLocalsHead].pPathStack=-1;
                                        }
                                    }
                                  else
                                    bOutOfMemory=TRUE;
                                }
                            }
                          if (! bReenter)
                            {
                              pFindPathLocals=pFindPathLocalsHead;
                              pFindPathLocalsHead=g_FindPathLocal[pFindPathLocals].pNext;
                              --g_nFindPathLocals;
                              pFindPathLocals=-1;
                            }
                        }
                      else
                        bOutOfMemory=TRUE;
                    }
                  while ((! bOutOfMemory) && bReenter);
              }
            if (! bOutOfMemory)
              {
                nReturnAddress=g_FindPathParameter[pFindPathParametersHead].nReturnAddress;
                pFindPathParameters=pFindPathParametersHead;
                pFindPathParametersHead=g_FindPathParameter[pFindPathParameters].pNext;
                --g_nFindPathParameters;
                pFindPathParameters=-1;
                bResume=TRUE;
              }
          }
        while ((! bOutOfMemory) && nReturnAddress && (nIterations < 1000));
      }
    else
      bOutOfMemory=TRUE;
    g_nPaths=0;
    g_nFindPathLocals=0;
    g_nFindPathParameters=0;
    return;
  }

static void WayOut(
 int16 nRooms,
 int16 pRoom,
 int16 pRoomEntrance,
 int16 *ppRoomWithName,
 int8 nTreasures,
 int8 *bErr)
  {
    int8  bCarryingTreasure;
    int16 nRoom;
    int8  nTreasure;
    char  *szText;
    char  *szWayOut;

    if (pRoom == pRoomEntrance)
      DisplayText("You\'re already at the entrance.",bErr);
    else
      {
        bCarryingTreasure=FALSE;
        for (nTreasure=nTreasures; ((! bCarryingTreasure) && (nTreasure--));)
          bCarryingTreasure=(g_Treasure[nTreasure].pRoomTreasure == -1);
        if (bCarryingTreasure)
          {
            szWayOut=NULL;
            FindPathToEntrance(pRoom,pRoomEntrance,&szWayOut);
            if (szWayOut)
              {
                do
                  nRoom=RandomNumber()%nRooms;
                while (ppRoomWithName[nRoom] == pRoom);
                g_Treasure[nTreasure].pRoomTreasure=ppRoomWithName[nRoom];
                if (1+StrLen("The pirate takes one of your treasures.  "
                 "As he leaves, he shouts the letters \"")
                 +StrLen(szWayOut)+StrLen("\".") < SENTENCE_LEN)
                  {
                    szText=&g_szMsg[0];
                    StrCpy(szText,"The pirate takes one of your treasures.  ");
                    if (StrLen(szWayOut) == 1)
                      StrCat(szText,"As he leaves, he shouts the letter \"");
                    else
                      StrCat(szText,"As he leaves, he shouts the letters \"");
                    StrCat(szText,szWayOut);
                    StrCat(szText,"\".");
                    DisplayText(szText,bErr);
                  }
                else
                  {
                    DisplayText("Fatal error:  memory was exhausted at locaton 6.",bErr);
                    *bErr=TRUE;
                  }
              }
            else
              DisplayText("Nothing happens.",bErr);
          }
        else
          DisplayText("Nothing happens.",bErr);
      }
    return;
  }

static void HideTreasuresAndWeapons(
 int16 nRooms,
 int16 *ppRoomWithName,
 int8  nTreasures)
  {
    int8  bChokepointFound;
    int16 nRoom;
    int8  nTreasure;

    for (nTreasure=0; nTreasure < nTreasures; ++nTreasure)
      {
        bChokepointFound=FALSE;
        nRoom=0;
        while ((! bChokepointFound) && (nRoom < nRooms))
          if (g_Room[ppRoomWithName[nRoom]].nChokepoint == nTreasure)
            bChokepointFound=TRUE;
          else
            ++nRoom;
        if (bChokepointFound)
          {
            do
              nRoom=RandomNumber()%nRooms;
            while (g_Room[ppRoomWithName[nRoom]].nChokepoint != nTreasure);
            g_Treasure[nTreasure].pRoomTreasure=ppRoomWithName[nRoom];
            do
              nRoom=RandomNumber()%(nRooms+1);
            while (g_Room[ppRoomWithName[nRoom]].nChokepoint >= nTreasure);
            g_Treasure[nTreasure].pRoomWeapon=ppRoomWithName[nRoom];
          }
        else
          {
            nRoom=RandomNumber()%nRooms;
            g_Treasure[nTreasure].pRoomTreasure=ppRoomWithName[nRoom];
            g_Treasure[nTreasure].pRoomWeapon=ppRoomWithName[nRooms];
          }
      }
    return;
  }

static intC Command(
 void)
  {
    intC nResult;

    printf("Command? ");
    nResult=ReadChar();
    return nResult;
  }

static void ListPassages(
 int16 pRoom,
 int8  *bErr)
  {
    int8  nDirection;
    intC  nFirstChar;
    int16 nPassage;
    int16 nPassages;
    int16 pRoomAlongPassage;
    int16 pTreasure;
    char  *szDescription;
    char  szPassages [] = "There are passages North, South, East, West, Up, and Down.";
    char  *szText;

    nPassages=0;
    for (nDirection=6; nDirection--;)
      if ((g_Room[pRoom].Adjacent.pPassage[nDirection] != -1)
      &&  (! g_Passage[g_Room[pRoom].Adjacent.pPassage[nDirection]].bBlocked))
        ++nPassages;
    if (nPassages > 1)
      StrCpy(&szPassages[0],"There are passages");
    else
      StrCpy(&szPassages[0],"There is a passage");
    nPassage=0;
    for (nDirection=0; nDirection < 6; ++nDirection)
      if ((g_Room[pRoom].Adjacent.pPassage[nDirection] != -1)
      &&  (! g_Passage[g_Room[pRoom].Adjacent.pPassage[nDirection]].bBlocked))
        {
          if (++nPassage > 1)
            if (nPassages != 2)
              StrCat(&szPassages[0],",");
          StrCat(&szPassages[0]," ");
          if (nPassage == nPassages)
            if (nPassages > 1)
              StrCat(&szPassages[0],"and ");
          StrCat(&szPassages[0],g_szDirection[nDirection]);
        }
    StrCat(&szPassages[0],".");
    DisplayText(&szPassages[0],bErr);
    for (nDirection=0; (! *bErr) && (nDirection < 6); ++nDirection)
      {
        szDescription=NULL;
        pRoomAlongPassage=pRoom;
        while ((! *bErr)
        && (g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection] != -1)
        && (! g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].bBlocked)
        && (szDescription == NULL))
          {
            if ((pTreasure=g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian) != -1)
              {
                nFirstChar=ToUpper((intC) *(g_Treasure[pTreasure].szGuardian));
                if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                  if (1+StrLen("The passage ")+StrLen(g_szDirection[nDirection])+StrLen(" is guarded by an ")+StrLen(g_Treasure[pTreasure].szGuardian)+StrLen(".") < SENTENCE_LEN)
                    {
                      szText=&g_szMsg[0];
                      StrCpy(szText,"The passage ");
                      StrCat(szText,g_szDirection[nDirection]);
                      StrCat(szText," is guarded by an ");
                      StrCat(szText,g_Treasure[pTreasure].szGuardian);
                      StrCat(szText,".");
                      DisplayText(szText,bErr);
                    }
                  else
                    {
                      DisplayText("Fatal error:  memory was exhausted at locaton 7.",bErr);
                      *bErr=TRUE;
                    }
                else
                  if (1+StrLen("The passage ")+StrLen(g_szDirection[nDirection])+StrLen(" is guarded by a ")+StrLen(g_Treasure[pTreasure].szGuardian)+StrLen(".") < SENTENCE_LEN)
                    {
                      szText=&g_szMsg[0];
                      StrCpy(szText,"The passage ");
                      StrCat(szText,g_szDirection[nDirection]);
                      StrCat(szText," is guarded by a ");
                      StrCat(szText,g_Treasure[pTreasure].szGuardian);
                      StrCat(szText,".");
                      DisplayText(szText,bErr);
                    }
                  else
                    {
                      DisplayText("Fatal error:  memory was exhausted at locaton 8.",bErr);
                      *bErr=TRUE;
                    }
              }
            if (g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pRoom1 == pRoomAlongPassage)
              pRoomAlongPassage=g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pRoom2;
            else
              pRoomAlongPassage=g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pRoom1;
            szDescription=g_Room[pRoomAlongPassage].szDescription;
          }
      }
    return;
  }

static void ListTreasuresAndWeapons(
 int16 pRoom,
 int8  nTreasures,
 int8  *bErr)
  {
    int16 nBytes;
    intC  nFirstChar;
    int8  nItem;
    int8  nItems;
    int8  nTreasure;
    char  *szText;

    nItems=0;
    for (nTreasure=nTreasures; nTreasure--;)
      {
        if (g_Treasure[nTreasure].pRoomTreasure == pRoom)
          ++nItems;
        if (g_Treasure[nTreasure].pRoomWeapon == pRoom)
          ++nItems;
      }
    if (nItems)
      {
        nBytes=0;
        if (nItems == 1)
          nBytes+=StrLen("There is ");
        else
          nBytes+=StrLen("There are ");
        nItem=0;
        for (nTreasure=nTreasures; nTreasure--;)
          {
            if (g_Treasure[nTreasure].pRoomTreasure == pRoom)
              {
                nFirstChar=ToUpper((intC) *(g_Treasure[nTreasure].szTreasure));
                ++nItem;
                if (nItem < nItems)
                  {
                    if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                      nBytes+=StrLen("an ");
                    else
                      nBytes+=StrLen("a ");
                    nBytes+=StrLen(g_Treasure[nTreasure].szTreasure);
                    if (nItems == 2)
                      nBytes+=StrLen(" ");
                    else
                      nBytes+=StrLen(", ");
                  }
                else
                  {
                    if (nItems > 1)
                      nBytes+=StrLen("and ");
                    if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                      nBytes+=StrLen("an ");
                    else
                      nBytes+=StrLen("a ");
                    nBytes+=StrLen(g_Treasure[nTreasure].szTreasure);
                    nBytes+=StrLen(" here.");
                  }
              }
            if (g_Treasure[nTreasure].pRoomWeapon == pRoom)
              {
                nFirstChar=ToUpper((intC) *(g_Treasure[nTreasure].szWeapon));
                ++nItem;
                if (nItem < nItems)
                  {
                    if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                      nBytes+=StrLen("an ");
                    else
                      nBytes+=StrLen("a ");
                    nBytes+=StrLen(g_Treasure[nTreasure].szWeapon);
                    if (nItems == 2)
                      nBytes+=StrLen(" ");
                    else
                      nBytes+=StrLen(", ");
                  }
                else
                  {
                    if (nItems > 1)
                      nBytes+=StrLen("and ");
                    if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                      nBytes+=StrLen("an ");
                    else
                      nBytes+=StrLen("a ");
                    nBytes+=StrLen(g_Treasure[nTreasure].szWeapon);
                    nBytes+=StrLen(" here.");
                  }
              }
          }
        if (nBytes < SENTENCE_LEN)
          {
            szText=&g_szMsg[0];
            if (nItems == 1)
              StrCpy(szText,"There is ");
            else
              StrCpy(szText,"There are ");
            nItem=0;
            for (nTreasure=nTreasures; nTreasure--;)
              {
                if (g_Treasure[nTreasure].pRoomTreasure == pRoom)
                  {
                    nFirstChar=ToUpper((intC) *(g_Treasure[nTreasure].szTreasure));
                    ++nItem;
                    if (nItem < nItems)
                      {
                        if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                          StrCat(szText,"an ");
                        else
                          StrCat(szText,"a ");
                        StrCat(szText,g_Treasure[nTreasure].szTreasure);
                        if (nItems == 2)
                          StrCat(szText," ");
                        else
                          StrCat(szText,", ");
                      }
                    else
                      {
                        if (nItems > 1)
                          StrCat(szText,"and ");
                        if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                          StrCat(szText,"an ");
                        else
                          StrCat(szText,"a ");
                        StrCat(szText,g_Treasure[nTreasure].szTreasure);
                        StrCat(szText," here.");
                      }
                  }
                if (g_Treasure[nTreasure].pRoomWeapon == pRoom)
                  {
                    nFirstChar=ToUpper((intC) *(g_Treasure[nTreasure].szWeapon));
                    ++nItem;
                    if (nItem < nItems)
                      {
                        if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                          StrCat(szText,"an ");
                        else
                          StrCat(szText,"a ");
                        StrCat(szText,g_Treasure[nTreasure].szWeapon);
                        if (nItems == 2)
                          StrCat(szText," ");
                        else
                          StrCat(szText,", ");
                      }
                    else
                      {
                        if (nItems > 1)
                          StrCat(szText,"and ");
                        if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                          StrCat(szText,"an ");
                        else
                          StrCat(szText,"a ");
                        StrCat(szText,g_Treasure[nTreasure].szWeapon);
                        StrCat(szText," here.");
                      }
                  }
              }
            DisplayText(szText,bErr);
          }
        else
          {
            DisplayText("Fatal error:  memory was exhausted at location 22.",bErr);
            *bErr=TRUE;
          }
      }
    return;
  }

static void Carry(
 int16 pRoom,
 int8  nTreasures,
 int8  *bErr)
  {
    int8 nPickedUp;
    int8 nTreasure;

    nPickedUp=0;
    for (nTreasure=nTreasures; nTreasure--;)
      {
        if (g_Treasure[nTreasure].pRoomWeapon == pRoom)
          {
            ++nPickedUp;
            g_Treasure[nTreasure].pRoomWeapon=-1;
          }
        if (g_Treasure[nTreasure].pRoomTreasure == pRoom)
          {
            ++nPickedUp;
            g_Treasure[nTreasure].pRoomTreasure=-1;
          }
      }
    if (nPickedUp == 0)
      DisplayText("There is nothing to carry.",bErr);
    return;
  }

static void Leave(
 int16 pRoom,
 int8  nTreasures,
 int8  *bErr)
  {
    int8 nLeft;
    int8 nTreasure;

    nLeft=0;
    for (nTreasure=nTreasures; nTreasure--;)
      if (g_Treasure[nTreasure].pRoomTreasure == -1)
        {
          ++nLeft;
          g_Treasure[nTreasure].pRoomTreasure=pRoom;
        }
    if (nLeft == 0)
      DisplayText("You aren\'t carrying anything to leave.",bErr);
    return;
  }

static void Inventory(
 int16 nChokepoints,
 int8  nTreasures,
 int8  *bErr)
  {
    int16 nBytes;
    intC  nFirstChar;
    int8  nItem;
    int8  nItems;
    int8  nTreasure;
    char  *szText;

    nItems=0;
    for (nTreasure=nTreasures; nTreasure--;)
      if (g_Treasure[nTreasure].pRoomTreasure == -1)
        ++nItems;
    for (nTreasure=nChokepoints; nTreasure--;)
      if ((g_Treasure[nTreasure].pRoomWeapon == -1) && (! g_Treasure[nTreasure].bSlain))
        ++nItems;
    if (nItems)
      {
        nBytes=0;
        nItem=0;
        szText=&g_szMsg[0];
        nBytes+=StrLen("You are carrying ");
        for (nTreasure=nTreasures; nTreasure--;)
          if (g_Treasure[nTreasure].pRoomTreasure == -1)
            {
              nFirstChar=ToUpper((intC) *(g_Treasure[nTreasure].szTreasure));
              ++nItem;
              if (nItem < nItems)
                {
                  if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                    nBytes+=StrLen("an ");
                  else
                    nBytes+=StrLen("a ");
                  nBytes+=StrLen(g_Treasure[nTreasure].szTreasure);
                  if (nItems == 2)
                    nBytes+=StrLen(" ");
                  else
                    nBytes+=StrLen(", ");
                }
              else
                {
                  if (nItems > 1)
                    nBytes+=StrLen("and ");
                  if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                    nBytes+=StrLen("an ");
                  else
                    nBytes+=StrLen("a ");
                  nBytes+=StrLen(g_Treasure[nTreasure].szTreasure);
                  nBytes+=StrLen(".");
                }
            }
        for (nTreasure=nChokepoints; nTreasure--;)
          if ((g_Treasure[nTreasure].pRoomWeapon == -1) && (! g_Treasure[nTreasure].bSlain))
            {
              nFirstChar=ToUpper((intC) *(g_Treasure[nTreasure].szWeapon));
              ++nItem;
              if (nItem < nItems)
                {
                  if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                    nBytes+=StrLen("an ");
                  else
                    nBytes+=StrLen("a ");
                  nBytes+=StrLen(g_Treasure[nTreasure].szWeapon);
                  if (nItems == 2)
                    nBytes+=StrLen(" ");
                  else
                    nBytes+=StrLen(", ");
                }
              else
                {
                  if (nItems > 1)
                    nBytes+=StrLen("and ");
                  if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                    nBytes+=StrLen("an ");
                  else
                    nBytes+=StrLen("a ");
                  nBytes+=StrLen(g_Treasure[nTreasure].szWeapon);
                  nBytes+=StrLen(".");
                }
            }
        if (nBytes < SENTENCE_LEN)
          {
            nItem=0;
            szText=&g_szMsg[0];
            StrCpy(szText,"You are carrying ");
            for (nTreasure=nTreasures; nTreasure--;)
              if (g_Treasure[nTreasure].pRoomTreasure == -1)
                {
                  nFirstChar=ToUpper((intC) *(g_Treasure[nTreasure].szTreasure));
                  ++nItem;
                  if (nItem < nItems)
                    {
                      if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                        StrCat(szText,"an ");
                      else
                        StrCat(szText,"a ");
                      StrCat(szText,g_Treasure[nTreasure].szTreasure);
                      if (nItems == 2)
                        StrCat(szText," ");
                      else
                        StrCat(szText,", ");
                    }
                  else
                    {
                      if (nItems > 1)
                        StrCat(szText,"and ");
                      if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                        StrCat(szText,"an ");
                      else
                        StrCat(szText,"a ");
                      StrCat(szText,g_Treasure[nTreasure].szTreasure);
                      StrCat(szText,".");
                    }
                }
            for (nTreasure=nChokepoints; nTreasure--;)
              if ((g_Treasure[nTreasure].pRoomWeapon == -1) && (! g_Treasure[nTreasure].bSlain))
                {
                  nFirstChar=ToUpper((intC) *(g_Treasure[nTreasure].szWeapon));
                  ++nItem;
                  if (nItem < nItems)
                    {
                      if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                        StrCat(szText,"an ");
                      else
                        StrCat(szText,"a ");
                      StrCat(szText,g_Treasure[nTreasure].szWeapon);
                      if (nItems == 2)
                        StrCat(szText," ");
                      else
                        StrCat(szText,", ");
                    }
                  else
                    {
                      if (nItems > 1)
                        StrCat(szText,"and ");
                      if ((nFirstChar == 'A') || (nFirstChar == 'E') || (nFirstChar == 'I') || (nFirstChar == 'O') || (nFirstChar == 'U'))
                        StrCat(szText,"an ");
                      else
                        StrCat(szText,"a ");
                      StrCat(szText,g_Treasure[nTreasure].szWeapon);
                      StrCat(szText,".");
                    }
                }
            DisplayText(szText,bErr);
          }
        else
          {
            DisplayText("Fatal error:  memory was exhausted at location 21.",bErr);
            *bErr=TRUE;
          }
      }
    else
      DisplayText("You aren\'t carrying anything.",bErr);
    return;
  }

static void Move(
 int16 *nMoves,
 int8  nDirection,
 int16 *pRoom,
 int8  *bErr)
  {
    int8  bGuarded;
    int16 pRoomAlongPassage;
    char  *szDescription;
    char  *szText;

    if ((g_Room[(*pRoom)].Adjacent.pPassage[nDirection] != -1)
    && (! g_Passage[g_Room[(*pRoom)].Adjacent.pPassage[nDirection]].bBlocked))
      {
        bGuarded=FALSE;
        szDescription=NULL;
        pRoomAlongPassage=*pRoom;
        while ((! *bErr)
        &&     (g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection] != -1)
        &&     (szDescription == NULL))
          {
            if (g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian != -1)
              if (g_Treasure[g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian].pRoomWeapon != -1)
                if (1+StrLen("You carry nothing to overcome the ")
                 +StrLen(g_Treasure[g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian].szGuardian)+StrLen(".") < SENTENCE_LEN)
                  {
                    szText=&g_szMsg[0];
                    bGuarded=TRUE;
                    StrCpy(szText,"You carry nothing to overcome the ");
                    StrCat(szText,g_Treasure[g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian].szGuardian);
                    StrCat(szText,".");
                    DisplayText(szText,bErr);
                  }
                else
                  {
                    DisplayText("Fatal error:  memory was exhausted at locaton 15.",bErr);
                    *bErr=TRUE;
                  }
              else
                if (1+StrLen("Your ")
                 +StrLen(g_Treasure[g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian].szWeapon)
                 +StrLen(" overcomes the ")
                 +StrLen(g_Treasure[g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian].szGuardian)+StrLen(".") < SENTENCE_LEN)
                  {
                    szText=&g_szMsg[0];
                    StrCpy(szText,"Your ");
                    StrCat(szText,g_Treasure[g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian].szWeapon);
                    StrCat(szText," overcomes the ");
                    StrCat(szText,g_Treasure[g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian].szGuardian);
                    StrCat(szText,".");
                    DisplayText(szText,bErr);
                    g_Treasure[g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian].bSlain=TRUE;
                    g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pTreasureGuardian=-1;
                  }
                else
                  {
                    DisplayText("Fatal error:  memory was exhausted at locaton 16.",bErr);
                    *bErr=TRUE;
                  }
            else;
            if (g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pRoom1 == pRoomAlongPassage)
              pRoomAlongPassage=g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pRoom2;
            else
              pRoomAlongPassage=g_Passage[g_Room[pRoomAlongPassage].Adjacent.pPassage[nDirection]].pRoom1;
            szDescription=g_Room[pRoomAlongPassage].szDescription;
          }
        if (! bGuarded)
          {
            ++(*nMoves);
            g_Room[pRoomAlongPassage].bVisited=TRUE;
            *pRoom=pRoomAlongPassage;
          }
      }
    else
      DisplayText("You can\'t go that way.",bErr);
    return;
  }

static void Points(
 int16 nRooms,
 int16 pRoomEntrance,
 int16 *ppRoomWithName,
 int8 nTreasures,
 int16 nMoves,
 int8 bRank,
 int8 *bErr)
  {
    int16 nRoom;
    int16 nRoomsVisited;
    int16 nScore;
    int8  nTreasure;
    int16 nTreasuresCarried;
    int16 nTreasuresRecovered;
    char  szMoves [12];
    char  szRooms [12];
    char  szRoomsVisited [12];
    char  szScore [12];
    char  *szText;
    char  szTreasures [12];
    char  szTreasuresCarried [12];
    char  szTreasuresRecovered [12];

    IToA(nRooms+1,&szRooms[0]);
    nScore=0;
    nTreasuresRecovered=0;
    nTreasuresCarried=0;
    nRoomsVisited=0;
    for (nRoom=0; nRoom <= nRooms; ++nRoom)
      if (g_Room[ppRoomWithName[nRoom]].bVisited)
        ++nRoomsVisited;
    IToA(nRoomsVisited,&szRoomsVisited[0]);
    IToA(nMoves,&szMoves[0]);
    if (1+StrLen("You have moved ")+StrLen(&szMoves[0])+StrLen(" times to visit ")+StrLen(&szRoomsVisited[0])+StrLen(" of ")+StrLen(&szRooms[0])+StrLen(" locations.") < SENTENCE_LEN)
      {
        szText=&g_szMsg[0];
        StrCpy(szText,"You have moved ");
        StrCat(szText,&szMoves[0]);
        StrCat(szText," times to visit ");
        StrCat(szText,&szRoomsVisited[0]);
        StrCat(szText," of ");
        StrCat(szText,&szRooms[0]);
        StrCat(szText," locations.");
        DisplayText(szText,bErr);
      }
    else
      {
        DisplayText("Fatal error:  memory was exhausted at locaton 17.",bErr);
        *bErr=TRUE;
      }
    if (! *bErr)
      {
        IToA(nTreasures,&szTreasures[0]);
        for (nTreasure=nTreasures; nTreasure--;)
          if (g_Treasure[nTreasure].pRoomTreasure == -1)
            ++nTreasuresCarried;
        IToA(nTreasuresCarried,&szTreasuresCarried[0]);
        if (1+StrLen("You hold ")+StrLen(&szTreasuresCarried[0])+StrLen(" of ")+StrLen(&szTreasures[0])+StrLen(" treasures.") < SENTENCE_LEN)
          {
            szText=&g_szMsg[0];
            StrCpy(szText,"You hold ");
            StrCat(szText,&szTreasuresCarried[0]);
            StrCat(szText," of ");
            StrCat(szText,&szTreasures[0]);
            StrCat(szText," treasures.");
            DisplayText(szText,bErr);
          }
        else
          {
            DisplayText("Fatal error:  memory was exhausted at locaton 18.",bErr);
            *bErr=TRUE;
          }
      }
    if (! *bErr)
      {
        for (nTreasure=nTreasures; nTreasure--;)
          if (g_Treasure[nTreasure].pRoomTreasure == pRoomEntrance)
            ++nTreasuresRecovered;
        IToA(nTreasuresRecovered,&szTreasuresRecovered[0]);
        if (1+StrLen("You have returned ")+StrLen(&szTreasuresRecovered[0])+StrLen(" of ")+StrLen(&szTreasures[0])+StrLen(" treasures to the entrance of the mine.") < SENTENCE_LEN)
          {
            szText=&g_szMsg[0];
            StrCpy(szText,"You have returned ");
            StrCat(szText,&szTreasuresRecovered[0]);
            StrCat(szText," of ");
            StrCat(szText,&szTreasures[0]);
            StrCat(szText," treasures to the entrance of the mine.");
            DisplayText(szText,bErr);
          }
        else
          {
            DisplayText("Fatal error:  memory was exhausted at locaton 19.",bErr);
            *bErr=TRUE;
          }
      }
    if (! *bErr)
      {
        nScore=25*nRoomsVisited/(nRooms+1)+75*nTreasuresRecovered/nTreasures+45*nTreasuresCarried/nTreasures;
        if (nRoomsVisited > 5*nRooms)
          {
            nScore=nScore-nRoomsVisited/(5*nRooms);
            if (nScore < 0)
              nScore=0;
          }
        IToA(nScore,&szScore[0]);
        if (1+StrLen("You have scored ")+StrLen(&szScore[0])+StrLen(" of 100 points.") < SENTENCE_LEN)
          {
            szText=&g_szMsg[0];
            StrCpy(szText,"You have scored ");
            StrCat(szText,&szScore[0]);
            StrCat(szText," of 100 points.");
            DisplayText(szText,bErr);
          }
        else
          {
            DisplayText("Fatal error:  memory was exhausted at locaton 20.",bErr);
            *bErr=TRUE;
          }
      }
    if (! *bErr)
      {
        if (bRank)
          {
            if (nScore < 25)
              DisplayText("Your score ranks you as a beginner.",bErr);
            else if (nScore < 50)
              DisplayText("Your score ranks you as a novice adventurer.",bErr);
            else if (nScore < 75)
              DisplayText("Your score ranks you as a seasoned explorer.",bErr);
            else if (nScore < 100)
              DisplayText("Your score ranks you as a grissly old prospector.",bErr);
            else
              DisplayText("Your score ranks you as an expert treasure hunter; there is no higher rating.",bErr);
          }
      }
    return;
  }

static void PlayGame(
 int16 nRooms,
 int16 pRoomEntrance,
 int16 *ppRoomWithName,
 int16 nChokepoints,
 int8  nTreasures,
 int8  *bErr)
  {
    intC  nCommand;
    int16 nMoves;
    int16 pRoom;

    nMoves=0;
    pRoom=pRoomEntrance;
    nCommand=(int) 'Q';
    clear_screen();
    g_nLinesWritten=0;
    g_szParagraph[0]='\0';
    do
      {
        DisplayText(g_Room[pRoom].szDescription,bErr);
        if (! *bErr)
          ListTreasuresAndWeapons(pRoom,nTreasures,bErr);
        if (! *bErr)
          ListPassages(pRoom,bErr);
        if (! *bErr)
          {
            DisplayParagraph(&g_szParagraph[0]);
            nCommand=Command();
            clear_screen();
            g_nLinesWritten=0;
            g_szParagraph[0]='\0';
            switch (nCommand)
              {
                case (int) 'N':
                case (int) 'n':
                  Move(&nMoves,0,&pRoom,bErr);
                  break;
                case (int) 'S':
                case (int) 's':
                  Move(&nMoves,1,&pRoom,bErr);
                  break;
                case (int) 'E':
                case (int) 'e':
                  Move(&nMoves,2,&pRoom,bErr);
                  break;
                case (int) 'W':
                case (int) 'w':
                  Move(&nMoves,3,&pRoom,bErr);
                  break;
                case (int) 'U':
                case (int) 'u':
                  Move(&nMoves,4,&pRoom,bErr);
                  break;
                case (int) 'D':
                case (int) 'd':
                  Move(&nMoves,5,&pRoom,bErr);
                  break;
                case (int) 'C':
                case (int) 'c':
                  Carry(pRoom,nTreasures,bErr);
                  break;
                case (int) 'I':
                case (int) 'i':
                  Inventory(nChokepoints,nTreasures,bErr);
                  break;
                case (int) 'L':
                case (int) 'l':
                  Leave(pRoom,nTreasures,bErr);
                  break;
                case (int) 'P':
                case (int) 'p':
                  Points(nRooms,pRoomEntrance,ppRoomWithName,nTreasures,nMoves,FALSE,bErr);
                  break;
                case (int) 'O':
                case (int) 'o':
                  WayOut(nRooms,pRoom,pRoomEntrance,ppRoomWithName,nTreasures,bErr);
                  break;
                case (int) 'Q':
                case (int) 'q':
                  nCommand='Q';
                  Points(nRooms,pRoomEntrance,ppRoomWithName,nTreasures,nMoves,TRUE,bErr);
                  break;
                case (int) 'H':
                case (int) 'h':
                  DisplayHelp(bErr);
                  break;
                default:
                  DisplayText("I don\'t recognize that command.",bErr);
                  ListCommands(bErr);
                  break;
              }
          }
      }
    while ((! *bErr) && (nCommand != (int) 'Q'));
    DisplayParagraph(&g_szParagraph[0]);
    return;
  }

int main(
 void)
  {
    int8  bErr;
    int16 nChokepoints;
    int16 nRooms;
    int8  nTreasures;
    int16 pTreasure;
    int16 pRoomEntrance;
    int16 *ppRoomWithName;

    bErr=FALSE;
    g_szParagraph[0]='\0';
    g_nLinesPerPage=(int16) screen_height();
    g_nMaxLineLen=(int16) (screen_width()-1);
    InitializeGlobals();
    g_nRooms=0;
    pRoomEntrance=g_nRooms;
    ++g_nRooms;
    g_Room[pRoomEntrance].szDescription="You\'re in the entrance to the mine.";
    g_Room[pRoomEntrance].bMined=TRUE;
    g_Room[pRoomEntrance].nChokepoint=-1;
    g_Room[pRoomEntrance].bVisited=TRUE;
    g_Room[pRoomEntrance].Adjacent.Passage.pPassageNorth=-1;
    g_Room[pRoomEntrance].Adjacent.Passage.pPassageSouth=-1;
    g_Room[pRoomEntrance].Adjacent.Passage.pPassageEast=-1;
    g_Room[pRoomEntrance].Adjacent.Passage.pPassageWest=-1;
    g_Room[pRoomEntrance].Adjacent.Passage.pPassageUp=-1;
    g_Room[pRoomEntrance].Adjacent.Passage.pPassageDown=-1;
    SeedRandomNumberGenerator();
    nRooms=NUM_ROOMS;
    if (! bErr)
      ShuffleRoomDescriptions(nRooms,&g_szRoom[0]);
    nTreasures=NUM_TREASURES;
    pTreasure=0;
    nChokepoints=0;
    if (! bErr)
      {
        ppRoomWithName=&g_pRoomWithName[0];
        ExcavateMine(nRooms,pRoomEntrance,nTreasures,&nChokepoints,ppRoomWithName,&bErr);
        if (! bErr)
          HideTreasuresAndWeapons(nRooms,ppRoomWithName,nTreasures);
        if (! bErr)
          PlayGame(nRooms,pRoomEntrance,ppRoomWithName,nChokepoints,nTreasures,&bErr);
      }
    printf("Press any key to exit.");
    ReadChar();
    return bErr;
  }
